@@ -234,7 +234,8 @@ sub use_reproxy_backend {
         $extra_hdr .= "Host: $host\r\n";
     }
 
-    my $headers = "GET $datref->[2] HTTP/1.0\r\nConnection: keep-alive\r\n${extra_hdr}\r\n";
+    my $req_method = $self->{req_headers}->request_method eq 'HEAD' ? 'HEAD' : 'GET';
+    my $headers = "$req_method $datref->[2] HTTP/1.0\r\nConnection: keep-alive\r\n${extra_hdr}\r\n";
 
     $be->{req_headers} = Perlbal::HTTPHeaders->new(\$headers);
     $be->state('sending_req');
@@ -1071,6 +1072,16 @@ sub buffered_upload_update {
         return;
     }
 
+    # can't proceed if we have no disk file to async write to
+    # people reported seeing this crash rarely in production...
+    # must be a race between previously in-flight's write
+    # re-invoking a write immediately after something triggered
+    # a buffered upload purge.
+    unless ($self->{bufh}) {
+        $self->close;
+        return;
+    }
+
     # at this point, we want to do some writing
     my $bref = shift(@{$self->{read_buf}});
     my $len = length $$bref;
